from .model_stable_video_diffusion import SVD
from .model_stable_diffusion_2 import SD2
from .model_stable_diffusion_3 import SD3
from .model_stable_diffusion_upscaler import SDUpScaler
from .model_depth_anything_v2 import DepthAnythingV2
from .model_depth2image import Depth2Image
from .model_clip_interrogator import CLIPInterrogator
from .model_instruct_ir import InstructIR
from .model_controlnet_stable_diffusion_2 import ControlNetSD2
from .model_controlnet_stable_diffusion_3 import ControlNetSD3
from .model_flux_1_dev import Flux1Dev
